//+------------------------------------------------------------------+
//|  DMI + guide                      Editid from            DMI.mq4 |
//|           Original             Copyright  2006, Fernando Gomes. |
//|                                    http://www.ciavox.com/~fgomes |
//|         Edited part  Copyright  2010,  k-matsu                  |
//|                               http://www.age.jp/~k-matsu/FX/MT4/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, Fernando Gomes."
#property link      "http://www.ciavox.com/~fgomes"

//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 5
//#property  indicator_color1  Lime
#property  indicator_color1  Aqua
#property  indicator_color2  Red
//#property  indicator_color3  Yellow
#property  indicator_color3  DeepSkyBlue
//#property  indicator_color4  DeepSkyBlue
#property  indicator_color4  Yellow
//#property  indicator_color5  Blue
#property  indicator_color5  PaleGoldenrod

#property  indicator_minimum 0
#property  indicator_level1  20
#property  indicator_level2  40
#property  indicator_maximum 60

//---- indicator parameters
extern int Smooth   = 14;
extern bool Hide_DI_Plus  = False;
extern bool Hide_DI_Minus = False;
extern bool Hide_DX   = true;
extern bool Hide_ADX  = False;
extern bool Hide_ADXR = true;

extern color GuideText_colorNormal=White;
extern color GuideText_colorUp=GreenYellow;
extern color GuideText_colorDown=Coral;
extern color GuideText_colorStrong=Yellow;
extern color GuideText_colorWeak=DarkGray;
extern int TurningPoint_margin=2;
extern double ValueEqual_margin=0.1;

//---- indicator buffers
double b_di_p[];     // +DM
double b_di_m[];     // -DM
double b_dmi[];      // DX (DMI itself)
double b_adx[];      // Average Directional indeX
double b_adxr[];     // Average Directional indeX Rate

//---- non-indicator buffers
//---- NOTICE that these arrays must be included in ArraySetAsSeries()
double dm_p_avg[];
double dm_m_avg[];
double tr_avg[];

string obj_prefix="DMI+guide";


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init() {
   //---- indicator buffers mapping
   IndicatorBuffers(8);
   if (   !SetIndexBuffer(0,b_di_p)
       && !SetIndexBuffer(1,b_di_m)
       && !SetIndexBuffer(2,b_dmi)
       && !SetIndexBuffer(3,b_adx)
       && !SetIndexBuffer(4,b_adxr)
       && !SetIndexBuffer(5,dm_p_avg)
       && !SetIndexBuffer(6,dm_m_avg)
       && !SetIndexBuffer(7,tr_avg)
      ) { Print("cannot set indicator buffers!"); return(-1); }
   //---- name for DataWindow and indicator subwindow label
   IndicatorShortName("DMI("+Smooth+")");
   SetIndexLabel(0,"+DI");
   SetIndexLabel(1,"-DI");
   SetIndexLabel(2,"DX");
   SetIndexLabel(3,"ADX");
   SetIndexLabel(4,"ADXR");
   //----
   SetIndexStyle(0, DRAW_LINE+Hide_DI_Plus*DRAW_NONE, STYLE_SOLID, 1, indicator_color1);
   SetIndexStyle(1, DRAW_LINE+Hide_DI_Minus*DRAW_NONE, STYLE_SOLID, 1, indicator_color2);
   SetIndexStyle(2, DRAW_LINE+Hide_DX*DRAW_NONE, STYLE_SOLID, 1, indicator_color3);
   SetIndexStyle(3, DRAW_LINE+Hide_ADX*DRAW_NONE, STYLE_SOLID, 1, indicator_color4);
   SetIndexStyle(4, DRAW_LINE+Hide_ADXR*DRAW_NONE, STYLE_SOLID, 1, indicator_color5);
   
   //----
   SetIndexDrawBegin(0,Smooth);
   SetIndexDrawBegin(1,Smooth);
   SetIndexDrawBegin(2,Smooth);
   SetIndexDrawBegin(3,2*Smooth);
   SetIndexDrawBegin(4,3*Smooth);

   return(0);
}
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit() {
   // delete guides
   int objnameprefix_length=StringLen(obj_prefix);
   for(int i = ObjectsTotal() - 1; i >= 0; i--) {
     string label = ObjectName(i);
     if(StringSubstr(label, 0, objnameprefix_length) == obj_prefix)
       ObjectDelete(label);
   }

   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start() {
  int counted_bars=IndicatorCounted();
  if (counted_bars<0) return(-1);
  if (Bars < 2) return(-1);
  if (Smooth < 2) return(-1);
  int limit=Bars-1-counted_bars;
  
  double smooth = 1.0 / (Smooth * 1.0);

  int firstDMI  = Bars-1-Smooth;
  int firstADX  = Bars-1-(2*Smooth);
  int firstADXR = Bars-1-(3*Smooth);
  
  for (int i=limit; i>=0; i--) {
    double high;
    double low;
    double dm_p;
    double dm_m;
    
    if (i<firstDMI) {
      high = High[i]  - High[i+1];
      low  = Low[i+1] - Low[i];
      dm_p = 0.0;
      dm_m = 0.0;
      if ((high > low) && (high > 0.0))
        dm_p = high;
      else if ((low > high) && (low > 0.0))
        dm_m = low;
      // calculate averages (cumulative formula)
      dm_p_avg[i] = dm_p_avg[i+1] - ( smooth * dm_p_avg[i+1] ) + dm_p;
      dm_m_avg[i] = dm_m_avg[i+1] - ( smooth * dm_m_avg[i+1] ) + dm_m;
      tr_avg[i]   = tr_avg[i+1]   - ( smooth * tr_avg[i+1] )   + calcTR(i);
    } else if (i==firstDMI) {
      double sum_dm_p = 0.0;
      double sum_dm_m = 0.0;
      double sum_tr   = 0.0;
      for (int j=i; j<i+Smooth; j++) {
        high = High[j]  - High[j+1];
        low  = Low[j+1] - Low[j];
        dm_p = 0.0 ;
        dm_m = 0.0 ;
        if ((high > low) && (high > 0.0))
          dm_p = high;
        else if ((low > high) && (low > 0.0))
          dm_m = low;
        sum_dm_p = sum_dm_p + dm_p;
        sum_dm_m = sum_dm_m + dm_m;
        sum_tr   = sum_tr   + calcTR(j);
      }
      // define current values
      dm_p_avg[i] = sum_dm_p * smooth;
      dm_m_avg[i] = sum_dm_m * smooth;
      tr_avg[i]   = sum_tr   * smooth;
    } else {
      tr_avg[i] = 0.0;
    }

    // Calculate +DI and -DI
    if (tr_avg[i] > 0.0) {
      b_di_p[i] = 100.0 * dm_p_avg[i] / tr_avg[i];
      b_di_m[i] = 100.0 * dm_m_avg[i] / tr_avg[i];
    } else {
      b_di_p[i] = 0.0;
      b_di_m[i] = 0.0;
    }

    // calcule DMI
    double sum  = b_di_p[i] + b_di_m[i];
    double diff = MathAbs( b_di_p[i] - b_di_m[i] );
    if (sum > 0.0) {
        b_dmi[i] = 100.0 *  diff / sum;
    } else {
        b_dmi[i] = 0.0 ;
    }

    // Calculate ADX
    if (i<firstADX) {   
      b_adx[i] = smooth*(b_adx[i+1]*(Smooth-1) + b_dmi[i]);
    } else if (i==firstADX) {
      double sum_dmi = 0.0;
      for (int k=i+Smooth+1; k>i; k--) {
        sum_dmi = sum_dmi + b_dmi[k];
      }
      b_adx[i] = smooth * sum_dmi;
    } else {
      b_adx[i] = 0.0;
    }
    
    // Calculate ADXR
    if (i<=firstADXR) {   
      b_adxr[i] = (b_adx[i] + b_adx[i+Smooth]) / 2;
    } else {
      b_adxr[i] = 0.0;
    }
    
  } // for


   // KCh\yW()
   int Guide_ydis = 2;

   color GuideText_color = GuideText_colorNormal;

   // +DI-DI
   int DI_compare = compareArray(b_di_p, b_di_m, 0, 0, ValueEqual_margin);
   string DI_compare_text;
   switch(DI_compare) {
     case  2: DI_compare_text="+DI-DI:"; GuideText_color = GuideText_colorUp; break;
     case  1: DI_compare_text="+DI-DI㔲"; GuideText_color = GuideText_colorUp; break;
     case  0: DI_compare_text="+DI-DIl"; GuideText_color = GuideText_colorNormal; break;
     case -1: DI_compare_text="+DI-DI"; GuideText_color = GuideText_colorDown; break;
     case -2: DI_compare_text="+DI-DI:"; GuideText_color = GuideText_colorDown; break;
   }
   createGuideLabel("DI_compare", 1, 2, Guide_ydis, DI_compare_text, 8, "MS UI Gothic", GuideText_color);
   Guide_ydis+=10;

   // ADX
   int ADX_value = 0;
   string ADX_value_text;
   if(b_adx[0] >= 40) {
     ADX_value = 1;
     ADX_value_text="ADX >= 40(gh)";
     GuideText_color = GuideText_colorStrong;
   } else if(b_adx[0] <= 20) {
     ADX_value = -1;
     ADX_value_text="ADX <= 20(W(アgh))";
     GuideText_color = GuideText_colorWeak;
   } else {
     ADX_value = -0;
     ADX_value_text="20 < ADX < 40";
     GuideText_color = GuideText_colorNormal;
   }
   createGuideLabel("ADX_value", 1, 2, Guide_ydis, ADX_value_text, 8, "MS UI Gothic", GuideText_color);
   Guide_ydis+=10;

   // ADX̌X
   int ADX_Trend = getPointTrend(b_adx, 0, ValueEqual_margin);
   string ADX_Trend_text;
   switch(ADX_Trend) {
     case  1: ADX_Trend_text="ADX̌XF"; GuideText_color = GuideText_colorStrong; break;
     case  0: ADX_Trend_text="ADX̌XF"; GuideText_color = GuideText_colorNormal; break;
     case -1: ADX_Trend_text="ADX̌XF"; GuideText_color = GuideText_colorWeak; break;
   }
   // ADX̌X̕ω
   int ADX_Trend_Before = getPointTrend(b_adx, 1, ValueEqual_margin);
   if (ADX_Trend * ADX_Trend_Before == -1) {
     switch(ADX_Trend) {
       case  1: ADX_Trend_text="ADX̌XFɕω"; break;
       case -1: ADX_Trend_text="ADX̌XFɕω"; break;
     }
   }
   createGuideLabel("ADX-Trend", 1, 2, Guide_ydis, ADX_Trend_text, 8, "MS UI Gothic", GuideText_color);
   Guide_ydis+=10;

   // ADXADXR
   int ADX_ADXR_compare = compareArray(b_adx, b_adxr, 0, 0, ValueEqual_margin);
   string ADX_ADXR_compare_text;
   switch(ADX_ADXR_compare) {
     case  2: ADX_ADXR_compare_text="ADXADXR:"; GuideText_color = GuideText_colorStrong; break;
     case  1: ADX_ADXR_compare_text="ADXADXR㔲"; GuideText_color = GuideText_colorStrong; break;
     case  0: ADX_ADXR_compare_text="ADXADXRl"; GuideText_color = GuideText_colorNormal; break;
     case -1: ADX_ADXR_compare_text="ADXADXR"; GuideText_color = GuideText_colorWeak; break;
     case -2: ADX_ADXR_compare_text="ADXADXR:"; GuideText_color = GuideText_colorWeak; break;
   }
   createGuideLabel("ADX_ADXR_compare", 1, 2, Guide_ydis, ADX_ADXR_compare_text, 8, "MS UI Gothic", GuideText_color);
   Guide_ydis+=10;

   // ADXDI
   int ADX_plusDI_compare = compareArray(b_adx, b_di_p, 0, 0, ValueEqual_margin);
   int ADX_minusDI_compare = compareArray(b_adx, b_di_m, 0, 0, ValueEqual_margin);
   int plusDI_Trend = getPointTrend(b_di_p, 0, ValueEqual_margin);
   int minusDI_Trend = getPointTrend(b_di_m, 0, ValueEqual_margin);
   string ADX_DI_compare_text = " ";
   if ( (DI_compare > 0) && (minusDI_Trend < 0) && (ADX_minusDI_compare == 1) ) {
     ADX_DI_compare_text="ADX-DI㔲"; GuideText_color = GuideText_colorUp;
   } 
   if ( (DI_compare < 0) && (plusDI_Trend < 0) && (ADX_plusDI_compare == 1) ) {
     ADX_DI_compare_text="ADX+DI㔲"; GuideText_color = GuideText_colorDown;
   }
   createGuideLabel("ADX_DI_compare", 1, 2, Guide_ydis, ADX_DI_compare_text, 8, "MS UI Gothic", GuideText_color);
   if (ADX_DI_compare_text != "") Guide_ydis+=10;


  return(0);
}

double calcTR(int bar) {
  double a = High[bar] - Low[bar];              // A = Today's High - Today's Low
  double b = MathAbs(Close[bar+1] - High[bar]); // B = Yesterday's Close - Today's High
  double c = MathAbs(Close[bar+1] - Low[bar]);  // C = Yesterday's Close - Today's Low
  return(MathMax(MathMax(a,b),c));
}


// subroutine
int createGuideLabel(string subname, int corner, int xdistance, int ydistance, string text, int fontsize, string font, color textcolor)
{
  string objectname = obj_prefix+subname;
  ObjectCreate(objectname, OBJ_LABEL, WindowOnDropped(), 0, 0);
  ObjectSet(objectname, OBJPROP_CORNER,corner);
  ObjectSet(objectname, OBJPROP_XDISTANCE, xdistance);
  ObjectSet(objectname, OBJPROP_YDISTANCE, ydistance);
  ObjectSetText(objectname, text, fontsize, font, textcolor);
  return(0);
}

// return 1(up trend), -1(down trend), 0(flat)
int getPointTrend(double& array[],int shift, double margin)
{
   if (array[shift] - array[shift+1] > margin  &&  array[shift] - array[shift+2] > margin) {
     return(1);
   } else if (array[shift+1] - array[shift]  > margin && array[shift+2] - array[shift] > margin) {
     return(-1);
   } else {
     return(0);
   }
}

// return 2(the source is up to the target), 1( the source crosses over the target),
// 0(difference is under the margin),
// -1(the source crosses under the target), -2(the source is under to the target)
int compareArray(double& sourceArray[], double& targetArray[], int source_shift, int target_shift, double margin)
{
   int retval;
   if (sourceArray[source_shift] - targetArray[target_shift] > margin) {
     retval=2;
     for (int i=0; i<=TurningPoint_margin; i++) {
       if (targetArray[target_shift+i+1] - sourceArray[source_shift+i+1] >= margin 
        && targetArray[target_shift+i+2] - sourceArray[source_shift+i+2] > margin ) {
         retval=1; i=TurningPoint_margin;
       }
     }
   } else if (targetArray[target_shift] - sourceArray[source_shift] > margin ) {
     retval=-2;
     for (i=0; i<=TurningPoint_margin; i++) {
       if (sourceArray[source_shift+i+1] - targetArray[target_shift+i+1] >= margin
        && sourceArray[source_shift+i+2] - targetArray[target_shift+i+2] > margin) {
         retval=-1; i=TurningPoint_margin;
       }
     }
   } else {
     retval=0;
   }
   return(retval);
}

//+------------------------------------------------------------------+